"use client";
import useShowToast from "@/app/hooks/useShowToast";
import { editAttempt, storeAttempt } from "@/app/lib/attempt/action";
import { createConsumerRequest } from "@/app/lib/consumers/action";
import { ImageWrapper, InputWrapper, SelectWrapper, ToogleWrapper } from "@/components/fields/Fields";
import FormComponent from "@/components/form/Form";
import { getStorage, removeStorage, setStorage } from "@/utils/navigation";
import prepareValidationSchema from "@/utils/validation-schema";
import { Box, Button, FormControl, FormLabel, Text, useColorModeValue } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useEffect, useRef, useState } from "react";
import { ConsumerInterface } from "../type/consumer";
import { formFields } from "../variables/formFields";
import WebcamCapture from "./WebcamCapture";
import { createFileRequest } from "@/app/lib/kyc/action";
import { generateUniqueId, onCompleteImage, onUploadFiles } from "@/utils/common";
import { useFormikContext } from "formik";
import { AttemptedInterface } from "../../assignments/type/attempted";
import useNetworkStatus from "@/app/hooks/useNetworkStatus";
import { ImageUpload } from "@/types/attachment";
import { getAssignmentList } from "@/app/lib/assignment/action";
import { createLocationRequest } from "@/app/lib/attendance/action";
import ConditionalFieldMeter from "../../dc/components/ConditionalFieldMeter";

interface Location {
  latitude: string;
  longitude: string;
}

const ConsumerForm = ({ data, draft = false }: { data?: any; draft?: boolean }) => {
  const formRef = useRef(null);
  const isOnline = useNetworkStatus();
  const router = useRouter();
  const showToast = useShowToast();
  const textColor = useColorModeValue("navy.700", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");

  const [isCamera, setIsCamera] = useState(false);
  const [imageSrc, setImageSrc] = useState<string[]>([]);
  const [uploadImages, setUploadImages] = useState<ImageUpload[]>([]);
  const [loading, setLoading] = useState(false);
  const [location, setLocation] = useState<Location>({ latitude: "", longitude: "" });

  const { initialFormData, validationSchema } = prepareValidationSchema(formFields);

  const fetchLocation = () => {
    if (!navigator.geolocation) {
      showToast("Failed", "Geolocation is not supported by this browser", "error");
      return;
    }

    navigator.geolocation.getCurrentPosition(
      (position) => {
        setLocation({
          latitude: position.coords.latitude.toString(),
          longitude: position.coords.longitude.toString(),
        });
      },
      (error) => {
        !draft && handleLocationError(error);
      }
    );
  };

  const handleLocationError = (error: any) => {
    switch (error.code) {
      case error.PERMISSION_DENIED:
        showToast("Failed", "Location access denied. Please enable location permissions in your browser settings.", "error");
        break;
      case error.POSITION_UNAVAILABLE:
        showToast("Failed", "Location information is unavailable. Turn on GPS.", "error");
        break;
      case error.TIMEOUT:
        showToast("Failed", "The request to get user location timed out. Try again.", "error");
        break;
      default:
        showToast("Failed", "An unknown error occurred while fetching location.", "error");
    }
  };

  const handleAttemptUpdate = async (assignmentId: number, consumerId: number) => {
    setLoading(true);
    try {
      const response = await editAttempt(assignmentId, { consumer_id: consumerId });
      if (response) {
        removeStorage("attempted_update");
        router.push("/officer/assignments");
      }
    } catch (error: any) {
      showToast("Failed to update attempt", error.message, "error");
    } finally {
      setLoading(false);
    }
  };

  const handleCreateConsumerOffline = async (values: FormData) => {
    setLoading(true);
    try {
      const storedData = await getStorage("draft-consumers");
      const data = await getStorage("attempted_update");
      const newId = generateUniqueId();
      const consumersData = [...(storedData || []), { ...values, created_at: new Date(), updated_at: new Date(), case_id: data.case_id, batch_id: data.batch_id, assignment_id: data.assignment_id, case: data?.case, offline: true, files: imageSrc, id: newId }];
      setStorage("draft-consumers", structuredClone(consumersData));
      removeStorage("attempted_update");
      router.push("/officer/consumer-drafts");

      showToast("Success", "Data Saved Locally", "success");
    } catch (error: any) {
      showToast("Failed", "The assignment id field is required Failed to Create Locally", "error");
    } finally {
      setLoading(false);
    }
  };

  const handleCreateAttempt = async () => {
    setLoading(true);
    try {
      const payload = draft ? { assignment_id: data?.assignment_id, case_id: data.case_id, batch_id: data.batch_id, consumer_id: null, kyc_id: null } : await getStorage("attempted_update");

      const response = await storeAttempt({ ...payload });
      return response;
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
      return null;
    } finally {
      setLoading(false);
    }
  };

  // TODO: Make interface for form data
  const handleCreateConsumer = async (values: any, attempt: AttemptedInterface) => {
    setLoading(true);
    try {
      // const uploadedData = draft && uploadImages?.length > 0 ? await onUploadFiles(uploadImages) : imageSrc?.length > 0 ? await onCompleteImage(imageSrc) : data?.files?.length > 0 ? await onCompleteImage(data?.files) : "";
      let uploadedData;

      if (draft && uploadImages?.length > 0 && data?.files?.length > 0) {
        const uploadImagesResult = await onUploadFiles(uploadImages);
        const dataFilesResult = await onCompleteImage(data?.files);
        uploadedData = [...uploadImagesResult, ...dataFilesResult]; // Combine results
      } else if (draft && uploadImages?.length > 0) {
        uploadedData = await onUploadFiles(uploadImages);
      } else if (imageSrc?.length > 0) {
        uploadedData = await onCompleteImage(imageSrc);
      } else if (draft && data?.files?.length > 0) {
        uploadedData = await onCompleteImage(data?.files);
      } else {
        uploadedData = [];
      }

      const response = await createConsumerRequest({ ...values, files: uploadedData?.length > 0 ? uploadedData : [] });
      if (response) {
        removeStorage("attempted");
        if (attempt) await handleAttemptUpdate(attempt?.id, response?.data?.id);
        if (attempt) await createLocationRequest({
          latitude: values?.latitude,
          longitude: values?.longitude,
          status: "attempting",
          attempt_id: attempt?.id
        })
        if (draft) {
          const storedData = await getStorage("draft-consumers");
          const newData = storedData.filter((val: any) => {
            return val.assignment_id !== attempt?.assignment_id;
          })
          setStorage("draft-consumers", structuredClone(newData));
        }
        const res = await getAssignmentList({ 'pagination': 'false' });
        const updatedData = {
          last_sync_at: new Date().toISOString(),
          data: res?.data || [],
        };

        setStorage('assignments', updatedData);
        showToast("Success", response?.message, "success");
      }
    } catch (error: any) {
      showToast("Failed to create consumer", error.message, "error");
    } finally {
      setLoading(false);
    }
  };

  // TODO: Make interface for form data
  const handleSubmit = async (values: any) => {
    const localData = await getStorage("lat-lng");
    values.latitude = location?.latitude?.toString() || localData?.latitude?.toString();
    values.longitude = location?.longitude?.toString() || localData?.longitude?.toString();

    if (draft) {
      values.latitude = data?.latitude;
      values.longitude = data?.longitude;
    }

    if (values?.id) {
      delete values.id;
    }

    if (isOnline) {
      if (!values?.latitude || !values?.longitude) {
        fetchLocation();
      }
      const res = await handleCreateAttempt();

      res && handleCreateConsumer(values, res.data);
    } else {
      !data && handleCreateConsumerOffline(values);
    }
  };

  const handleUploadImages = async (images: ImageUpload[]) => {
    setUploadImages(images);
  };

  useEffect(() => {
    fetchLocation();
  }, []);

  console.log("data", data);

  return (
    <>
      <FormComponent
        ref={formRef}
        validationSchema={validationSchema}
        initialValues={data ? { ...data } : initialFormData}
        // enableReinitialize // Ensures the form reinitializes with updated initial values
        submitForm={handleSubmit}>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Contact Person &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper form fieldName="contact_person" id="contact_person" variant="auth" w="full" mb="2px" fontSize="sm" type="text" fontWeight="500" defaultValue={data?.contact_person ?? ""} disabled={!!data?.contact_person} />
          </FormControl>
          <FormControl w="50%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Mobile No &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper form fieldName="mobile_no" id="mobile_no" variant="auth" w="full" mb="2px" fontSize="sm" type="number" fontWeight="500" defaultValue={data?.mobile_no ?? ""} disabled={!!data?.mobile_no} />
          </FormControl>
        </Box>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Paid Amount &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper form fieldName="paid_amount" id="paid_amount" variant="auth" w="full" mb="2px" fontSize="sm" type="number" fontWeight="500" defaultValue={data?.paid_amount ?? ""} disabled={!!data?.paid_amount} />
          </FormControl>
          <FormControl w="50%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              DC Request &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="dc_request"
              id="dc_request"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              options={[
                { label: "Yes", value: "1" },
                { label: "No", value: "0" },
              ]}
              defaultValue={data?.dc_request ?? ""}
              disabled={!!data?.dc_request}
            />
          </FormControl>
        </Box>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Meter / Un Meter: &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="meter_un_meter"
              id="meter_un_meter"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              options={[
                { label: "Yes", value: "1" },
                { label: "No", value: "0" },
              ]}
              defaultValue={data?.meter_un_meter ?? ""}
              disabled={!!data?.meter_un_meter}
            />
          </FormControl>
        </Box>

        <ConditionalFieldMeter textColor={textColor} brandStars={brandStars} data={data} />

        <Box display="flex" gap={15}>
          <FormControl w="100%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Remark &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="remarks"
              id="remarks"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              options={[
                { label: "Referred to IBC | RIBC", value: "Referred to IBC | RIBC" },
                { label: "Agreed to Pay | ATP", value: "Agreed to Pay | ATP" },
                { label: "Address not found | ANF", value: "Address not found | ANF" },
                { label: "Not Agreed to Pay | NAP", value: "Not Agreed to Pay | NAP" },
                { label: "Premises Closed | PC", value: "Premises Closed | PC" },
                { label: "Bill Delivered | BD", value: "Bill Delivered | BD" },
                { label: "Follow up required | FR", value: "Follow up required | FR" },
                { label: "Premises Demolished | PD", value: "Premises Demolished | PD" },
                { label: "House Under Construction | HOC", value: "House Under Construction | HOC" },
                { label: "Already Paid | AP", value: "Already Paid | AP" },
                { label: "Court Case | CC", value: "Court Case | CC" },
              ]}
              defaultValue={data?.remarks ?? ""}
              disabled={!!data?.remarks}
            />
          </FormControl>
        </Box>

        <Box display="flex" gap={15}>
          <FormControl w="100%">
            <FormLabel fontSize="sm" fontWeight="500" color={textColor} mb="8px">
              Additional Remarks &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper form fieldName="additional_remarks" id="additional_remarks" variant="auth" w="full" mb="2px" fontSize="sm" type="text" fontWeight="500" defaultValue={data?.additional_remarks ?? ""} disabled={!!data?.additional_remarks} />
          </FormControl>
        </Box>

        {isCamera && <WebcamCapture setImageSrc={setImageSrc} onCaptureComplete={() => setIsCamera(false)} />}

        {!data && (
          <Button onClick={() => setIsCamera(true)} mb={3}>
            Upload Photo
          </Button>
        )}

        {data && (
          <ImageWrapper
            form={false}
            updateOnParent={(images) => {
              handleUploadImages(images);
            }}
            label="Upload From Gallery"
            id="filess"
            fieldName="filess"
          />
        )}

        {imageSrc?.length > 0 && (
          <Box display="flex" flexWrap="wrap" gap={4}>
            {imageSrc.map((src: string, index: number) => (
              <img key={index} src={src} alt={`Captured ${index}`} style={{ width: "150px", height: "120px", objectFit: "cover", borderRadius: 10 }} />
            ))}
          </Box>
        )}
        {data?.files?.length > 0 && (
          <Box display="flex" flexWrap="wrap" gap={4}>
            {data?.files?.map((src: string, index: number) => (
              <img key={index} src={src} alt={`Captured ${index}`} style={{ width: "150px", height: "120px", objectFit: "cover", borderRadius: 10 }} />
            ))}
          </Box>
        )}

        <Button variant="brand" onClick={() => formRef.current?.externalHandleSubmit()} isDisabled={loading} isLoading={loading} mt={4}>
          Create
        </Button>
      </FormComponent>
    </>
  );
};

export default ConsumerForm;